local iftracker = CreateFrame("Frame","IFTrackerContainer",UIParent)
iftracker:SetFrameStrata("BACKGROUND")
iftracker:SetAlpha(0.5)
iftracker:SetMinResize(32, 32)
iftracker:SetMovable(false)
iftracker:EnableMouse(false)
iftracker:SetResizable(false)

iftracker.BELOW_THREE_MSG = "CHILL"
iftracker.ABOVE_THREE_MSG = "ENGAGE"
iftracker.IDLE_MSG = "IDLE"
iftracker.IDLE_VALUE_MSG = "0"
iftracker.UP_ARROW = "^"
iftracker.DOWN_ARROW = "v"
iftracker.IDLE_ARROW = "-"

function iftracker:ManageDefaultSettings()
	if IFTrackerSettings == nil then
		IFTrackerSettings = {}
	end
	if IFTrackerSettings.point == nil then
		IFTrackerSettings.point = "CENTER"
	end
	if IFTrackerSettings.xOfs == nil then
		IFTrackerSettings.xOfs = 0
	end
	if IFTrackerSettings.yOfs == nil then
		IFTrackerSettings.yOfs = 0
	end
	if IFTrackerSettings.width == nil then
		IFTrackerSettings.width = 128
	end
	if IFTrackerSettings.height == nil then
		IFTrackerSettings.height = 192
	end
	if IFTrackerSettings.showNoCombat == nil then
		IFTrackerSettings.showNoCombat = "true"
	end
end

iftracker:SetScript("OnEvent", function(self,event,name)
	if event == "ADDON_LOADED" and name == "IFTracker" then
		self:ManageDefaultSettings()
		self:SetWidth(IFTrackerSettings.width)
		self:SetHeight(IFTrackerSettings.height)
		self:SetPoint(IFTrackerSettings.point,IFTrackerSettings.xOfs,IFTrackerSettings.yOfs)
		self:CreateFrames()
		self:ManageAddon()
	elseif event == "PLAYER_ENTERING_WORLD" or event == "PLAYER_LOGIN" or event == "PLAYER_TALENT_UPDATE" then
		self:ManageDefaultSettings()
		self:ManageAddon()
	elseif event == "PET_BATTLE_OPENING_START" then
		self:Hide()
	elseif event == "PET_BATTLE_CLOSE" then
		self:ManageAddon()
	end
end)

iftracker:RegisterEvent("ADDON_LOADED")
iftracker:RegisterEvent("PLAYER_ENTERING_WORLD")
iftracker:RegisterEvent("PLAYER_LOGIN")
iftracker:RegisterEvent("PLAYER_TALENT_UPDATE")
iftracker:RegisterEvent("PET_BATTLE_OPENING_START")
iftracker:RegisterEvent("PET_BATTLE_CLOSE")

function iftracker:ManageAddon(forceShow)
	local group = GetActiveSpecGroup()
	local _,name,_,selected = GetTalentInfo( 3, 3, group )
	if name == "Incanter's Flow" and selected == true and IFTrackerSettings.showNoCombat == "true" then
		self:Show()
	elseif name == "Incanter's Flow" and selected == true and forceShow then
		self:Show()
	else
		self:Hide()
	end
end

function iftracker:ResetSettings()
	IFTrackerSettings = nil
	self:ManageDefaultSettings()
	self:SetWidth(IFTrackerSettings.width)
	self:SetHeight(IFTrackerSettings.height)
	self:SetPoint(IFTrackerSettings.point,IFTrackerSettings.xOfs,IFTrackerSettings.yOfs)
	self:SizeAndPositionFrames()
	self:ManageAddon()
end

function iftracker:CreateFrames()
	local texture = self:CreateTexture(nil,"BACKGROUND")
	texture:SetTexture(0,0,0,.25)
	texture:SetAllPoints(iftracker)
	texture.texture = texture

	self.statusFrame = CreateFrame("Frame","IFTrackerStatusTextFrame",self)

	local status_texture = self.statusFrame:CreateTexture(nil,"BACKGROUND")
	status_texture:SetTexture(0,0,0,0.25)
	status_texture:SetAllPoints(self.statusFrame)
	self.statusFrame.texture = status_texture

	self.statusText = self.statusFrame:CreateFontString(nil, "OVERLAY", "GameFontRed")
	self.statusText:SetJustifyH("Left")
	self.statusText:SetText(self.IDLE_VALUE_MSG)
	self.statusText:SetTextColor(0.5, 0.5, 0.5, 1)

	self.arrowTextLeft = self.statusFrame:CreateFontString(nil, "OVERLAY", "GameFontRed")
	self.arrowTextLeft:SetJustifyH("Left")
	self.arrowTextLeft:SetText(self.IDLE_ARROW)
	self.arrowTextLeft:SetTextColor(0.5, 0.5, 0.5, 1)

	self.arrowTextRight = self.statusFrame:CreateFontString(nil, "OVERLAY", "GameFontRed")
	self.arrowTextRight:SetJustifyH("Right")
	self.arrowTextRight:SetText(self.IDLE_ARROW)
	self.arrowTextRight:SetTextColor(0.5, 0.5, 0.5, 1)

	self.iconFrame = CreateFrame("Frame","IFTrackerIconFrame",iftracker)
	self.iconFrame:SetResizable(true)

	local icon_texture = self.iconFrame:CreateTexture(nil,"BACKGROUND")
	icon_texture:SetTexture("Interface\\Icons\\Ability_Mage_IncantersAbsorbtion.blp")
	icon_texture:SetAllPoints(self.iconFrame)
	self.iconFrame.texture = icon_texture

	self.messageFrame = CreateFrame("Frame","IFTrackerMessageTextFrame",self.iconFrame)

	local message_texture = self.messageFrame:CreateTexture(nil,"BACKGROUND")
	message_texture:SetTexture(0,0,0,0.5)
	message_texture:SetAllPoints(self.messageFrame)
	self.messageFrame.texture = message_texture

	self.messageText = self.messageFrame:CreateFontString(nil, "OVERLAY", "GameFontRed")
	self.messageText:SetJustifyH("Left")
	self.messageText:SetText(self.IDLE_MSG)
	self.messageText:SetTextColor(0.5, 0.5, 0.5, 1)

	self:SizeAndPositionFrames()
	self:registerEvents()
end

function iftracker:SizeAndPositionFrames()
	self.statusFrame:SetWidth(self:GetWidth())
	self.statusFrame:SetHeight(self:GetHeight()/3)
	self.statusFrame:SetPoint("TOP",0,0)

	self.arrowTextLeft:SetFont("Fonts\\FRIZQT__.TTF", self:GetHeight()/3)
	self.arrowTextLeft:SetHeight(self:GetHeight()/3)
	self.arrowTextLeft:SetPoint("LEFT",self.statusFrame ,0,0)

	self.statusText:SetFont("Fonts\\FRIZQT__.TTF", self:GetHeight()/3)
	self.statusText:SetHeight(self:GetHeight()/3)
	self.statusText:SetPoint("CENTER",self.statusFrame ,0,0)

	self.arrowTextRight:SetFont("Fonts\\FRIZQT__.TTF", self:GetHeight()/3)
	self.arrowTextRight:SetHeight(self:GetHeight()/3)
	self.arrowTextRight:SetPoint("RIGHT",self.statusFrame ,0,0)

	self.iconFrame:SetWidth(self:GetWidth())
	self.iconFrame:SetHeight(self:GetWidth())
	self.iconFrame:SetPoint("TOP",0,-(self.statusFrame:GetHeight()))
	self.messageFrame:SetWidth(self:GetWidth())
	self.messageFrame:SetHeight(self:GetHeight()/3)
	self.messageFrame:SetPoint("CENTER",0,0)
	self.messageText:SetFont("Fonts\\FRIZQT__.TTF", self:GetHeight()/3)
	self.messageText:SetHeight(self:GetHeight()/3)
	self.messageText:SetPoint("CENTER",self.messageFrame ,0,0)
end

function iftracker:registerEvents()
	self:RegisterForDrag("LeftButton","RightButton")
	self:SetScript("OnDragStart", function(self,event)
		if event == "LeftButton" then
			self:StartMoving()
		else
			self:SetScript("OnUpdate", function(self,elapsed)
				self:SizeAndPositionFrames()
			end)
			self:StartSizing()
		end
	end)
	self:SetScript("OnDragStop", function(self)
		self:StopMovingOrSizing()
		self:SetScript("OnUpdate",nil)
		IFTrackerSettings.top = self:GetTop()
		IFTrackerSettings.left = self:GetLeft()
		local point, _, _, xOfs, yOfs = self:GetPoint()
		IFTrackerSettings.point = point
		IFTrackerSettings.xOfs = xOfs
		IFTrackerSettings.yOfs = yOfs
		IFTrackerSettings.width = self:GetWidth()
		IFTrackerSettings.height = self:GetHeight()
	end)

	local stackHandler = CreateFrame("Frame","IFTrackEventHandler",self)

	stackHandler.isActive = false
	stackHandler.updateInterval = 1.0
	stackHandler.lastUpdateTime = 0
	stackHandler.lastStack = 1
	stackHandler.frameReference = self
	stackHandler.statusText = self.statusText
	stackHandler.messageText = self.messageText
	stackHandler.arrowTextLeft = self.arrowTextLeft
	stackHandler.arrowTextRight = self.arrowTextRight
	stackHandler.BELOW_THREE_MSG = self.BELOW_THREE_MSG
	stackHandler.ABOVE_THREE_MSG = self.ABOVE_THREE_MSG
	stackHandler.IDLE_MSG = self.IDLE_MSG
	stackHandler.IDLE_VALUE_MSG = self.IDLE_VALUE_MSG
	stackHandler.UP_ARROW = self.UP_ARROW
	stackHandler.DOWN_ARROW = self.DOWN_ARROW
	stackHandler.IDLE_ARROW = self.IDLE_ARROW

	stackHandler:RegisterEvent("PLAYER_REGEN_ENABLED")
	stackHandler:RegisterEvent("PLAYER_REGEN_DISABLED")

	stackHandler:SetScript("OnUpdate", function(self,elapsed)
		if self.isActive then
			self.lastUpdateTime = self.lastUpdateTime + elapsed
			if (self.lastUpdateTime > self.updateInterval) then
				local name,_,_,count= UnitBuff("player", "Incanter's Flow");
				if name and count then
					local delta = count - self.lastStack
					self.lastStack = count
					self.statusText:SetText(count)
					if delta < 0 then
						self.arrowTextLeft:SetTextColor(1, 0, 0, 1)
						self.arrowTextLeft:SetText(self.DOWN_ARROW)
						self.arrowTextRight:SetTextColor(1, 0, 0, 1)
						self.arrowTextRight:SetText(self.DOWN_ARROW)
					elseif delta > 0 then
						self.arrowTextLeft:SetTextColor(0, 1, 0, 1)
						self.arrowTextLeft:SetText(self.UP_ARROW)
						self.arrowTextRight:SetTextColor(0, 1, 0, 1)
						self.arrowTextRight:SetText(self.UP_ARROW)
					end
					if delta < 0 and count <= 3 then
						self.statusText:SetTextColor(1, 0, 0, 1)
						self.messageText:SetTextColor(1, 0, 0, 1)
						self.messageText:SetText(self.BELOW_THREE_MSG)
					elseif delta > 0  and count >= 3 then
						self.statusText:SetTextColor(0, 1, 0, 1)
						self.messageText:SetTextColor(0, 1, 0, 1)
						self.messageText:SetText(self.ABOVE_THREE_MSG)
					end
					self.lastUpdateTime = 0;
				end
			end
		end
	end)

	stackHandler:SetScript("OnEvent", function(self,event)
		if event == "PLAYER_REGEN_DISABLED" then
			self.isActive = true
			self.messageText:SetTextColor(1, 0, 0, 1)
			self.statusText:SetTextColor(1, 0, 0, 1)
			self.arrowTextLeft:SetTextColor(1,0,0,1)
			self.arrowTextRight:SetTextColor(1,0,0,1)
			stackHandler.frameReference:SetAlpha(1.0)
			self.frameReference:ManageAddon(true)
		else
			self.isActive = false
			self.messageText:SetText(self.IDLE_MSG)
			self.statusText:SetText(self.IDLE_VALUE_MSG)
			self.messageText:SetTextColor(0.5, 0.5, 0.5, 1)
			self.statusText:SetTextColor(0.5, 0.5, 0.5, 1)
			self.arrowTextLeft:SetTextColor(0.5, 0.5, 0.5, 1)
			self.arrowTextRight:SetTextColor(0.5, 0.5, 0.5, 1)
			self.arrowTextLeft:SetText(self.IDLE_ARROW)
			self.arrowTextRight:SetText(self.IDLE_ARROW)
			stackHandler.frameReference:SetAlpha(0.5)
			self.frameReference:ManageAddon()
		end
	end)

	SLASH_IFTracker1, SLASH_IFTracker2,SLASH_IFTracker3 = "/ift", "/ift lock","/ift unlock","/ift showNoCombat";
	function SlashCmdList.IFTracker(msg)
		local command, rest = msg:match("^(%S*)%s*(.-)$");
		if command == "lock" then
	    	self:SetMovable(false)
			self:EnableMouse(false)
			self:SetResizable(false)
			self:ManageAddon()
	    elseif command == "unlock" then
	    	self:SetMovable(true)
			self:EnableMouse(true)
			self:SetResizable(true)
			self:Show()
		elseif command == "showNoCombat" then
			IFTrackerSettings.showNoCombat = rest
			if self:IsResizable() == false then
				self:ManageAddon()
			end
		elseif command == "reset" then
			self:ResetSettings()
		else
			print("|cFF3399FFIFTracker commands|r");
			print("|cFF3399FF/ift|r lock")
			print("|cFF3399FF/ift|r unlock [left click drag to move. right click drag to size.]")
			print("|cFF3399FF/ift|r showNoCombat true|false [hide frame out of combat.]")
			print("|cFF3399FF/ift|r reset")
	    end 
	end
end